﻿namespace iFileProxy.Middleware
{
    using iFileProxy.Helpers;
    using iFileProxy.Models;
    using Microsoft.AspNetCore.Http;
    using System;
    using System.Collections.Generic;
    using System.Text.Json;
    using System.Threading.Tasks;

    public class IPAccessLimitMiddleware
    {
        private readonly RequestDelegate _next;
        private readonly Dictionary<string, Dictionary<string, uint>> _IPAccessCountDict;
        private readonly int _dailyRequestLimitPerIP;

        public IPAccessLimitMiddleware(RequestDelegate next, Dictionary<string, Dictionary<string, uint>> IPAccessCountDict, int dailyRequestLimitPerIP)
        {
            _next = next;
            _IPAccessCountDict = IPAccessCountDict;
            _dailyRequestLimitPerIP = dailyRequestLimitPerIP;
        }

        public async Task InvokeAsync(HttpContext context)
        {
            string dateStr = DateTime.Now.ToString("yyyy-MM-dd");
            string ipStr = MasterHelper.GetClientIPAddr(context);
            
            if (_IPAccessCountDict.ContainsKey(dateStr))
            {
                if (_IPAccessCountDict[dateStr].ContainsKey(ipStr))
                {
                    if (_IPAccessCountDict[dateStr][ipStr] >= _dailyRequestLimitPerIP)
                    {
                        context.Response.StatusCode = 200;
                        context.Response.ContentType = "application/json";
                        await context.Response.WriteAsync(JsonSerializer.Serialize(new CommonRsp { retcode = 403, message = "请求次数超过限制!" }));
                        return;
                    }
                    _IPAccessCountDict[dateStr][ipStr]++;
                }
                else
                {
                    _IPAccessCountDict[dateStr].Add(ipStr, 1);
                }
            }
            else
            {
                _IPAccessCountDict.Add(dateStr, new Dictionary<string, uint> { { ipStr, 1 } });
            }

            await _next(context);
        }
    }
}
